#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from onf_website.reference.insee.schema import ICommuneField, ICommunesListField
from onf_website.reference.insee.zmi.interfaces import ICommuneWidget, ICommunesListWidget
from pyams_form.interfaces.form import IFormLayer
from z3c.form.interfaces import IDataConverter, IFieldWidget

# import packages
from onf_website.reference.insee.model import Commune
from pyams_form.widget import widgettemplate_config
from pyams_utils.adapter import adapter_config
from z3c.form.browser.widget import HTMLInputWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.widget import Widget, FieldWidget
from zope.interface import implementer_only


#
# Commune field
#

@adapter_config(context=(ICommuneField, ICommuneWidget), provides=IDataConverter)
class CommuneDataConverter(BaseDataConverter):
    """Commune data converter"""

    def toWidgetValue(self, value):
        if not value:
            return None
        return Commune.find_by_insee_code(value).first()

    def toFieldValue(self, value):
        return value


@widgettemplate_config(mode='input', template='templates/commune-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/commune-display.pt', layer=IFormLayer)
@implementer_only(ICommuneWidget)
class CommuneWidget(HTMLInputWidget, Widget):
    """Commune widget"""

    @property
    def value_map(self):
        if not self.value:
            return ''
        return json.dumps({self.value.token: self.value.title})


@adapter_config(context=(ICommuneField, IFormLayer), provides=IFieldWidget)
def CommuneFieldWidget(field, request):
    """Commune field widget factory"""
    return FieldWidget(field, CommuneWidget(request))


#
# Communes list field
#

@adapter_config(context=(ICommunesListField, ICommunesListWidget), provides=IDataConverter)
class CommunesListDataConverter(BaseDataConverter):
    """Communes list data converter"""

    def toWidgetValue(self, value):
        if not value:
            return ()
        return Commune.find_by_insee_code(value).all()

    def toFieldValue(self, value):
        if not value:
            return []
        return value.split('|')


@widgettemplate_config(mode='input', template='templates/communes-list-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/communes-list-display.pt', layer=IFormLayer)
@implementer_only(ICommunesListWidget)
class CommunesListWidget(HTMLInputWidget, Widget):
    """Communes list widget"""

    @property
    def values(self):
        return '|'.join((commune.token for commune in self.value or ()))

    @property
    def values_map(self):
        result = {}
        [result.update({value.token: value.title}) for value in self.value or ()]
        return json.dumps(result)


@adapter_config(context=(ICommunesListField, IFormLayer), provides=IFieldWidget)
def CommunesListFieldWidget(field, request):
    """Communes list field widget factory"""
    return FieldWidget(field, CommunesListWidget(request))
