/*
 * ONF website ZMI extensions
 */

/* globals L, PyAMS_GIS */

(function($, globals) {

	'use strict';

	var MyAMS = globals.MyAMS;

	var ONF_website = {

		/**
		 * Location management
		 */

		location: {

			/* Switch location setting visibility */
			switchVisibility: function(element) {
				return function() {
					var source = $(this);
					var para = source.parents('tr');
					var container = para.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/switch-location-setting-visibility.json',
						{object_name: para.data('ams-element-name')},
						function(result, status) {
							if (result.visible) {
								$('i', source).attr('class', 'fa fa-fw fa-eye');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
							}
						});
				};
			},

			/* Switch location setting folding */
			switchUnfolded: function(element) {
				return function() {
					var source = $(this);
					var para = source.parents('tr');
					var container = para.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/switch-location-setting-folding.json',
						{object_name: para.data('ams-element-name')},
						function(result, status) {
							if (result.unfolded) {
								$('i', source).attr('class', 'fa fa-fw fa-folder-open-o');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-folder-o');
							}
						});
				};
			},

			/* Move location map marker to given position on filter selection */
			moveMapMarkerTo: function(event, options) {
				if (event.added) {
					var source = $(this);
					var map = source.parents('.map-header')
									.siblings('.map');
					var fieldname = map.data('map-leaflet-fieldname');
					var projection = $('select[name="' + fieldname + '.widgets.projection:list"]');
					MyAMS.jsonrpc.post('getLocationCoords', {
											source: event.added,
											to_srid: projection.val()
									   }, {
											url: '/api/onf_website/json'
									   }, function(data) {
											var result = data.result;
											PyAMS_GIS.position.moveMarkerTo(map, result.position, result.srid);
											var leafmap = map.data('leafmap');
											var layer = map.data('json-layer');
											if (layer !== undefined) {
												layer.remove();
											}
											layer = L.geoJSON(result.geometry, {
												style: {
													color: '#dd0000',
													weight: 3,
													opacity: 0.7
												}
											});
											layer.addTo(leafmap);
											map.data('json-layer', layer);
											leafmap.fitBounds(layer.getBounds());
									   });
				}
			}
		},


		/**
		 * Hearing management
		 */

		hearing: {

			/* Switch hearing setting visibility */
			switchVisibility: function(element) {
				return function() {
					var source = $(this);
					var para = source.parents('tr');
					var container = para.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/switch-hearing-setting-visibility.json',
						{object_name: para.data('ams-element-name')},
						function(result, status) {
							if (result.visible) {
								$('i', source).attr('class', 'fa fa-fw fa-eye');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
							}
						});
				};
			},

			/* Switch hearing setting folding */
			switchUnfolded: function(element) {
				return function() {
					var source = $(this);
					var para = source.parents('tr');
					var container = para.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/switch-hearing-setting-folding.json',
						{object_name: para.data('ams-element-name')},
						function(result, status) {
							if (result.unfolded) {
								$('i', source).attr('class', 'fa fa-fw fa-folder-open-o');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-folder-o');
							}
						});
				};
			},
		}
	};

	/**
	 * Location map listener to 'marker.changed' event
	 * If marker position is overriding a forest, get forest geometry and display it
	 * as GeoJSON, and update filter area with selected forest name
	 */
	$(document).on('marker.changed', function(event, map, position) {
		map = $(map);
		var mapdata = map.data();
		var fieldname = mapdata.mapLeafletFieldname;
		var srid = $('select[name="' + fieldname + '.widgets.projection:list"]');
		MyAMS.jsonrpc.post('getForestAt', {
			position: position,
			srid: srid.val()
		}, {
			url: '/api/onf_website/json'
		}, function(data) {
			// Reset map layer
			var leafmap = mapdata.leafmap;
			var layer = mapdata.jsonLayer;
			if (layer !== undefined) {
				layer.remove();
			}
			var filter = $('input[name="pyams_gis.map.header_location"]');
			var result = data.result;
			if (result) {
				// Update location filter
				filter.select2('data', result);
				// Update map with selected geometry
				layer = L.geoJSON(result.geometry, {
					style: {
						color: '#00dd00',
						weight: 3,
						opacity: 0.7
					}
				});
				layer.addTo(leafmap);
				map.data('json-layer', layer);
			} else {
				filter.select2('data', null);
			}
		});
	});

	/**
	 * Location map listener to 'marker.closed.position' event
	 * If marker selection dialog is closed with 'OK' button, if filter is not empty
	 * and if there is no already selected forest, update forests selection area with
	 * filter data
	 */
	$(document).on('marker.closed.position', function(event, options) {
		var filter = $('input[name="pyams_gis.map.header_location"]');
		if (filter.val()) {
			var forests = $('input[name="form.widgets.forests"]');
			if (forests && !forests.val()) {
				forests.select2('data', filter.select2('data'));
			}
		}
	});

	globals.ONF_website = ONF_website;

})(jQuery, this);
